#!/usr/bin/awk -f

# usage: gerb2ngc_point.awk [<*.gbr>] > <*.ngc>

BEGIN {
  ZOFF = "0.03937";	# -1mm
  ZON = "-0.007874";	# -0.2mm

  SCALE = 1.0e-4;
  mode = 0;
  check = 0;
  num = 0;
}

{
  gsub(/\r$/, "", $0);

  if ($0 == "%FSLAX24Y24*%") check = 1;
  if ($0 ~ /G36\*/) mode = 1;
  if ($0 ~ /G37\*/) mode = 0;
  if ($0 ~ /\%LPD\*/) mode = 2;
  if ($0 ~ /\%LPC\*/) mode = 0;
  if (mode == 1 && match($0, /^(X[0-9]+)(Y[0-9]+)D02\*(X[0-9]+)?(Y[0-9]+)?D01\*$/, tmp) != 0) {
    x0 = substr(tmp[1], 2) * SCALE;
    y0 = substr(tmp[2], 2) * SCALE;
    if (check == 1) check = 2;
  }
  if (mode == 0 && match($0, /(X[0-9]+)?(Y[0-9]+)?D03\*$/, tmp) != 0) {
    if (tmp[1] == "") {
      x1 = x1;
    } else {
      x1 = substr(tmp[1], 2) * SCALE - x0;
    }
    if (tmp[2] == "") {
      y1 = y1;
    } else {
      y1 = substr(tmp[2], 2) * SCALE - y0;
    }

    if (dup[x1, y1] == "") {
      dup[x1, y1] = 1;
      data_x1[num] = x1;
      data_y1[num] = y1;
      num++;
    }
  }
}

END {
  # Reversal (X-axis)
  x_min = data_x1[0];
  x_max = data_x1[0];
  for (i = 0; i < num; i++) {
    if (data_x1[i] < x_min) x_min = data_x1[i];
    if (data_x1[i] > x_max) x_max = data_x1[i];
  }
  x_off = x_min + x_max;
  for (i = 0; i < num; i++) {
    data_x1[i] = -data_x1[i] + x_off;
  }

  if (check != 2) {
    printf "ERROR\n" > "/dev/stderr";
    exit 1;
  }

  printf "%\r\n";
  printf "(Generated by gbr2ngc_point.awk)\r\n";
  printf "G17G20G40G49G54G80G90G94\r\n";
  printf "G1F60\r\n";

  x = 0.0;
  y = 0.0;
  while (num > 0) {
    for (i = 0; i < num; i++) {
      d = (data_x1[i] - x) * (data_x1[i] - x) + (data_y1[i] - y) * (data_y1[i] - y);
      if (i == 0 || d < m) {
	m = d;
	n = i;
      }
    }

    x1 = data_x1[n];
    y1 = data_y1[n];
    data_x1[n] = data_x1[num - 1];
    data_y1[n] = data_y1[num - 1];
    num--;

    printf "Z%.4f\r\n", ZOFF;
    printf "X%.4fY%.4f\r\n", x1, y1;
    printf "Z%.4f\r\n", ZON;

    x = x1;
    y = y1;
  }

  printf "Z%.4f\r\n", ZOFF;
  printf "X%.4fY%.4f\r\n", 0.0, 0.0;
  printf "Z%.4f\r\n", 0.0;
  printf "%\r\n";

}
